% This is a template for adaptive residual subsampling
% method for radial basis function 2-D interpolation problem.
%
% For reference, see:
% Adaptive residual subsampling methods for radial basis function
% interpolation and collocation problems. submitted to Computers Math. Appl
%
% Tobin A. Driscoll and Alfa R.H. Heryudono    05/14/2006
% MATLAB 7 is recommended.
%
% This basic template also performs boundary adaptation. The domain is a  
% square [-1 1]x[-1 1].
%
% Send any bugs report to : heryudon@math.udel.edu.

clear all;close all;
% Test function:
% Franke Functions
f = @(x,y) exp(-0.1*(x.^2 + y.^2)) ...
    + exp(-5*((x - 0.5).^2 + (y - 0.5).^2)) ...
    + exp(-15*((x+0.2).^2 + (y+0.4).^2)) ...
    + exp(-9*((x+0.8).^2 + (y-0.8).^2));
% Alternative test functions
%f = @(x,y) exp(-60*((x-0.35).^2+(y-0.25).^2))+0.2;
%f = @(x,y) -0.4*tanh(20*x.*y)+0.6;

% Adaptive residual subsampling parameters
thetar = 1e-3; thetac = 1e-5*thetar;
n = 2^4; % (must be even)
topc = n; % can be tweaked.

% Corner centers (never coarsened)
xc = [1 1;-1 1;-1 -1;1 -1];
cc = topc*ones(4,1); % corner shape parameter

% Initial discretization
ibox = initboxes2d(n);
N = length(ibox);

x = cat(1,ibox.midpt);
p1 = plot(x(:,1),x(:,2),'ko','MarkerSize',3.25,'MarkerFaceColor','k');
axis([-1 1 -1 1 0 2]); axis equal;
xlabel('x');ylabel('y');zlabel('z')
hold on; grid on;
tp = title('','erasemode','xor'); 

% Create first generation of "residual boxes." of interior points
[ibox,newres] = dorefine2d(ibox,true(N,1),1);

center = false(size(ibox));  center(1:N) = true;
residual = false(size(ibox));  residual(newres) = true;

% Begin main adaptive iteration
flag = true;
while any(flag)
  N = sum(center);  M = sum(residual);
  centerbox = ibox(center); residbox = ibox(residual);

  x = cat(1,centerbox.midpt);
  c = topc*2.^(cat(1,centerbox.depth));
  w = cat(1,residbox.midpt);
  
  x = [x;xc];
  c = [c;cc];
  
  % Update plots
  set(tp,'string',sprintf('N = %5d',N));
  set(p1,'Xdata',x(:,1),'Ydata',x(:,2));
  drawnow
 
  [A,B] = deal(zeros(N+4),zeros(M,N+4)); 
  for j=1:N+4
      A(:,j) = mq2d(x,x(j,:),c(j));
      B(:,j) = mq2d(w,x(j,:),c(j));
  end

  condition_number = cond(A)
  lambda = A\f(x(:,1),x(:,2)); 
  resid = B*lambda - f(w(:,1),w(:,2));
 
  boxarea = cat(1,residbox.dimen);
  zerolength = boxarea == 0;
  boxarea(zerolength) = 1;
  boxarea = prod(boxarea,2);
 
  normerr = -ones(size(ibox));
  % Error (residual) indicator
  normerr(residual) = abs(resid); % Pointwise error Indicator
  %normerr(residual) = abs(resid).*boxarea; % Area-based error Indicator
 
  % Look for coarsening opportunities in the region.
  count = 0;
  for i = find(center(:)).'
    check = abs(normerr(ibox(i).child));
    if sum(check < thetac)==ibox(i).Nchild
      center(i) = false;
      residual(i) = true;
      residual(ibox(i).child) = false;
      count = count+1;
    end
  end
  fprintf('Removing %i centers and ',count)
    
  % Boxes needing refinement in the region.
  flag = normerr > thetar;
  newcen = find(flag);
  center(newcen) = true;
  residual(newcen) = false;
  [ibox,newbox] = dorefine2d(ibox,flag,0);
  center(newbox) = false;
  residual(newbox) = true;
   
  fprintf('Adding %i centers.\n',sum(flag))
  fprintf('------Press Enter------\n')
end

% Interpolate and plot the function in finer grid based on known 
% RBF center points and its values. 
Np = 61;
xp = linspace(-1,1,Np)';
[X,Y] = meshgrid(xp,xp); 
X=X(:);Y=Y(:); NX = length(X); 
x = cat(1,ibox(center).midpt); 
c = topc*2.^cat(1,ibox(center).depth); 
N = sum(center);
x = [x;xc];
c = [c;cc];
A = zeros(N+4); B = zeros(NX,N+4); 
for j=1:N+4
    A(:,j) = mq2d(x,x(j,:),c(j));
    B(:,j) = mq2d([X Y],x(j,:),c(j));
end
lambda = A\f(x(:,1),x(:,2)); 
Z = B*lambda;
IN = inpolygon(X,Y,xc(:,1),xc(:,2)); X(~IN)=NaN; Y(~IN)=NaN;
X = reshape(X,Np,Np); Y = reshape(Y,Np,Np); Z = reshape(Z,Np,Np);
set(tp,'string',sprintf('N = %3i',N));
surf(X,Y,Z); 
hidden off;
